/*  
    Summarize_TD_functional_region_labels_v2.js
    
    Script to summarize functional region labels from Talairach Daemon for a number of *.td.txt files.

	Required settings in Talairach Client Preferences: 
	- Uncheck 'Add record number to output'
	- Separator: Comma (if using CSV file)
      - Select 'Input from file'

    Input for Talairach Client:
    Either
    - VOI details file from line 12 onwards
    or
    - VOI details imported in a spreadsheet program, and saving only the columns with Talairach coordinates in a CSV file

	Example line from a resulting *.td.csv file:
	"4.0,-90.0,-4.0,Right Cerebrum,Occipital Lobe,Lingual Gyrus,Gray Matter,Brodmann area 17,";

	Format of tdfilenames.txt file (to write manually):

	<total nr of td files>
	<filename 1>
	<nr of lines/labels in file 1>
	..
	<filename n>
	<nr of lines/labels in file n>

	Example:
	
	3
	/Volumes/121106_s12_d_haptic_vs_baseline_q0_001_thresh50_voithr500_2707_split_2.td.txt
	2707
	/Volumes/121106_s12_d_haptic_vs_baseline_q0_001_thresh50_voithr500_657_split_3.td.txt
	657
	/Volumes/121106_s12_d_haptic_vs_baseline_q0_001_thresh50_voithr500_638_split_4.td.txt
	638

	For each file, a *.td_info.txt file is written with the number of voxels per region.

	HB, Brain Innovation, 09-08-2012, latest update: 08-11-2012
	v0.2
*/


// please edit this variable below for your own purposes ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

var filenamesfile = String("/Volumes/COMPATIBLE/Data/tdfilenames_example.txt");
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



function readTextFile(name) {

    var filecounter;    
    BrainVoyagerQX.PrintToLog("Reading: " + name);
       
    var filenamesfile = new QFile(name);     
    filenamesfile.open(new QIODevice.OpenMode(QIODevice.ReadOnly));
    var textstr = new QTextStream(filenamesfile); 
    var nroffiles = new Number(parseInt(textstr.readLine()));
	BrainVoyagerQX.PrintToLog("Number of files: " + nroffiles);
    var projfilenames = new Array(nroffiles);
    for (filecounter = 0; filecounter < nroffiles; filecounter++) {
	 		var arr2 = new Array(2);
       		var filename = textstr.readLine();
	 	 	var nroflinesLine = textstr.readLine();
			var nroflines = parseInt(nroflinesLine);
        		BrainVoyagerQX.PrintToLog(filename + ", nr of lines: " + nroflines);
			arr2[0] = filename;
			arr2[1] = nroflines;
        		projfilenames[filecounter] = arr2;
    }
     filenamesfile.close();
     return projfilenames;
}


// ~~~ executable part

var projfls = this.readTextFile(filenamesfile);
//BrainVoyagerQX.PrintToLog("Length of projfls: " + projfls.length);
this.run(projfls);

function run(projfls) {

var filenr;
for (filenr = 0; filenr < projfls.length; filenr++) {

	var filename = String(projfls[filenr][0]);
	var nlines = projfls[filenr][1]; 
	var rc = new RegionContainer();
	var line;
	var linenr = 0;
	BrainVoyagerQX.PrintToLog("Reading: " + filename);
	var file = new QFile(filename); 
	file.open(new QIODevice.OpenMode(QIODevice.ReadOnly)); 
	var textstr = new QTextStream(file); 
	var line = textstr.readLine();
	BrainVoyagerQX.PrintToLog(line);
	//while (!file.atEnd()) {
	//while (line != null) {
	while (linenr < nlines) {
		line = textstr.readLine(); 
	//	BrainVoyagerQX.PrintToLog(line);
		if (line != null)	rc.processLine(line);	
		linenr++;
	} 
	file.close(); 
	BrainVoyagerQX.PrintToLog(rc.printRegionContainer());
	writeToTextFile(filename, rc.printRegionContainer());
	BrainVoyagerQX.PrintToLog("Finished round: " + (filenr + 1) + " of " + projfls.length);
}

}

// ~~~~ defined functions

// constructor for RegionContainer object
function RegionContainer() {

	// member variable
	this.regions = new Array();
	this.undefinedRegions = 0;

	// member functions
	this.processLine = processLine;
	this.addRegion = addRegion;
	this.containsRegion = containsRegion;
	this.getRegionIndex = getRegionIndex;
	this.getNrOfRegions = getNrOfRegions;
	this.addCoordinateToRegion = addCoordinateToRegion;
	this.printRegionContainer = printRegionContainer; 

} // end RegionContainer class

function processLine(line) {
	L = line.split(",");
	var region = new String(L[5]);
	var reg = new String(L[3]);
	var regi = new String(L[4]);
	if ((region == "*")|| (regi== "*") || (reg== "*")){
		this.undefinedRegions += 1;
	} else {
		if (!this.containsRegion(L[5])) {
			var r = new Region(L[5]);
			this.addRegion(r);
			BrainVoyagerQX.PrintToLog("Added region `" + r.getName() + "'");
		}
		this.addCoordinateToRegion(L[0], L[1], L[2], L[3],L[4],L[5],L[6],L[7],L[8]);
	}
}

function addRegion(regionObject) {
	this.regions.push(regionObject);
}

function containsRegion(regionName) {
		var doesContain = false;
		if (this.getRegionIndex(regionName) > -1) {
			doesContain = true;
		}
		return doesContain;
	}

function getRegionIndex(regionName) {
		var index = -1;
		for (i=0;i<this.regions.length;i++) {
			var objn = this.regions[i];
			var regionname = new String(objn.getName());
			//if (this.regions[i].getName().match(regionName) != null) {
			if (regionname.match(regionName) != null) {
				index = i;
				break;
			}
		}
		return index;
	}

function getNrOfRegions() {
		return this.regions.length;
	}

	function addCoordinateToRegion(talX,talY,talZ, hemisphere,lobe,gyrus,tissue,BrodmannArea) {
		// find the region
		if (this.containsRegion(gyrus)) {
			var regionNr = this.getRegionIndex(gyrus); 
			this.regions[regionNr].addCoordinate(talX,talY,talZ, hemisphere,lobe,gyrus,tissue,BrodmannArea); // address particular Region object
		}
		
	}

	function printRegionContainer() {
		var info = "~~~~~ \nNumber of regions: " + this.regions.length + "\n";
		for (i=0; i<this.regions.length; i++) {
			info += this.regions[i].printRegion() + "\n";
		}
		info += "Undefined region labels: " + this.undefinedRegions + "\n";
		info += "~~~~~\n";
		return info;
	}


// constructor for Region object
function Region(gyrus) {

	// member variables
	this.totalCoords = 0;
	this.talX = new Array(); // signed integer
	//var talY = new Array(); // signed integer
	//var talZ = new Array(); // signed integer
	this.LH = 0; // string (Left Cerebrum or Right Cerebrum) // level 1
	this.RH = 0;
	this.UH = 0;
	this.lobe;// = new Array(); // level 2
	this.name = gyrus; // level 3
	this.GM = 0; // GM or WM; level 4
	this.WM = 0;
	this.UM = 0; // unspecified matter
	this.BA = new Array(); // string, can contain several; level 5

	// member functions
	this.addCoordinate = addCoordinate;
	this.getName = getName;
	this.printRegion = printRegion;
	
} // end Region class


function addCoordinate(talX,talY,talZ, hemisphere,lobes,gyrus,tissuetype,BrodmannArea) { 

		// talX etc are currently ignored
		this.totalCoords += 1;

		// check hemisphere
		var hemi = new String(hemisphere);
		if (hemi.match("Left") != null) {
			this.LH += 1;
		} else if (hemi.match("Right") != null) {
			this.RH += 1;
		} else {
			this.UH += 1;
		} 

		this.hemisphere = hemi; // string (Left Cerebrum or Right Cerebrum)
		var tissue = new String(tissuetype);
		if (tissue.match("Gray") != null) {
			this.GM += 1;
		} else if (tissue.match("White") != null) {
			this.WM += 1;
		} else {
			this.UM += 1;
		}	 
		var area = new String(BrodmannArea);
		var tmp = area.split(" ");
		if (tmp.length == 3) {		
			var ba = parseInt(tmp[2]);
	//		BrainVoyagerQX.PrintToLog("BrodmannArea: " + ba);
			if (!contains(this.BA, ba)) {
				this.BA.push(ba); 
			}
		}
 }	

 function getName() {
		return this.name;
	}

function printRegion() {
		var regionInfo = "Name: " + this.getName() + "\n";
		regionInfo += "Number of coordinates: " + this.totalCoords + "\n";
		regionInfo += "LH: " + this.LH + ", RH: " + this.RH + ", hemisphere undefined: " + this.UH + "\n";
		regionInfo += "WM: " + this.WM + ", GM: " + this.GM + ", matter undefined: " + this.UM + "\n";
		regionInfo += "BA: " + this.BA.toString() + "\n";
		return regionInfo;
}

// check if array contains number 
function contains(arr, nr) {
	var doesContain = false;
	for (i=0; i<arr.length; i++) {
		if (arr[i] == nr) {
			doesContain = true;
			break;
		}
	}
	return doesContain;
}


function writeToTextFile(filename, text) {
	var filecounter; // write the textfile to the path of the first file 
	var name = this.getPath(filename) + getBasename(filename) + "_info.txt";
	BrainVoyagerQX.PrintToLog("Name of file: " + name);
	var filenamesfile = new QFile(name);
	 filenamesfile.open(new QIODevice.OpenMode(QIODevice.WriteOnly)); 
	var textstr = new QTextStream(filenamesfile); 
	textstr.writeString(text);
	filenamesfile.close(); 
	BrainVoyagerQX.PrintToLog("Wrote: " + name); 
} 

function getPath(filename) {
	var start = 0; // start searching from begin
 	var last = filename.lastIndexOf("/"); // search for last path separator 
	var path = filename.substring(start, last);//(last+1)); // now return path without filename 
	return path; 
}

function getBasename(filename) {
 	var start = filename.lastIndexOf("/"); // search for last path separator 
	var end = filename.lastIndexOf(".");
	var basename = filename.substring(start, (end+1)); // now return path without filename 
	return basename; 
}